Const ScreenX=800
Const Screeny=608
Const TileSize%=32
Const TileColumns%=4
Const TileRows%=4
Const TilesX%=ScreenX/TileSize
Const TilesY%=Screeny/TileSize
Const MapSizeX%=1024
Const MapSizeY%=512

Global Terrains%=(TileColumns*TileRows)
Global TileX%=256
Global TileY%=256
Global min%=255
Global max%=0

Local mode$="3D"

Graphics3D ScreenX,Screeny,32,2

SetBuffer BackBuffer()

Dim V0(TilesX,TilesY),V1(TilesX,TilesY),V2(TilesX,TilesY),V3(TilesX,TilesY)

start1=MilliSecs()

Dim Map(MapSizeX,MapSizeY)
bank=LoadBank("planet.raw",MapSizeX,MapSizeY)
For x=0 To MapSizeX-1
	For y=0 To MapSizeY-1
		Map(x,y)=GetBankPixel(bank,x,y,MapSizeX)
		If Map(x,y)>max Then max=Map(x,y)
		If Map(x,y)<min Then min=Map(x,y)
	Next
Next

tileimg=LoadImage("tiles.png")
tiletex=LoadTexture("tiles.png")
tiletexwidth=TextureWidth(tiletex)
tiletexheight=TextureHeight(tiletex)
ScaleTexture tiletex,tiletexwidth,tiletexheight

tilemap=CreateTileMap()
UpdateTileMap(tilemap,TileX,TileY)
EntityTexture tilemap,tiletex,0,0
EntityFX tilemap,1+2+32

Global Cam%=CreateCamera()
CameraRange Cam,1,ScreenX/2
PositionEntity Cam,0,0,-ScreenX/2

minimap=CreateMinimap(bank,MapSizeX,MapSizeY)

While Not KeyHit(1)
	
	Local sw%
	Local cx%=KeyDown(205)-KeyDown(203)
	Local cy%=KeyDown(208)-KeyDown(200)
	
	TileX=(TileX+cx) Mod MapSizeX
	If TileX<0 Then TileX=MapSizeX+TileX
	
	TileY=(TileY+cy)
	If TileY<0 Then TileY=0
	If TileY>MapSizeY-TilesY Then TileY=MapSizeY-TilesY
	
	If cx<>0 Or cy<>0 Then UpdateTileMap(tilemap,TileX,TileY)
	
	If KeyHit(57) Then
		sw=1-sw
		If sw=1 Then mode="2D" Else mode="3D"
	EndIf
	
	RenderWorld
	
	If sw=1 Then
		
		For y=0 To TilesY-1
			
			For x=0 To TilesX-1
				
				mx=(TileX+x) Mod MapSizeX
				my=(TileY+y)
				
				h=Normalize(Map(mx,my),min,max,0,Terrains)
				
				u=(h Mod TileColumns)
				v=(h/TileRows)
				
				DrawImageRect tileimg,x*32,y*32,u*32,v*32,32,32
				
			Next
			
		Next
		
	EndIf
	
	
	DrawImage minimap,0,0
	stepx=(MapSizeX/256)
	stepy=(MapSizeY/128)
	Rect TileX/stepx,TileY/stepy,TilesX/stepx,TilesY/stepy,0
	
	renders=renders+1
	
	If MilliSecs()-old_ms>=1000
		old_ms=MilliSecs()
		fps=renders
		renders=0
	EndIf
	
	Text 0,128,mode+": "+fps+" FPS"
	
	Flip 0
	
Wend

End

Function CreateMinimap(bank%,width%,height%)
	
	Local image%=CreateImage(256,128)
	Local buffer%=ImageBuffer(image)
	Local stepx%=width/256
	Local stepy%=height/128
	
	Local x%,y%,h%,rgb
	
	LockBuffer buffer
	
	For x=0 To 255
		
		For y=0 To 127
			
			h=Map(x*stepx,y*stepy)
			rgb=h*$10000+h*$100+h
			WritePixelFast x,y,rgb,buffer
			
		Next
		
	Next
	
	UnlockBuffer buffer
	
	Return image
	
End Function

Function Normalize#(value#=128.0,value_min#=0.0,value_max#=255.0,norm_min#=0.0,norm_max#=1.0)
	
	Return ((value-value_min)/(value_max-value_min))*(norm_max-norm_min)+norm_min
	
End Function

Function LoadBank(filename$,width%,height%)
	
	Local f%=OpenFile(filename$)
	
	Local bank%=CreateBank(width*height)
	
	ReadBytes(bank,f,0,BankSize(bank))
	
	CloseFile f
	
	Return bank
	
End Function

Function GetBankPixel(bank%,px%,py%,width%)
	
	Return PeekByte(bank,(py*width)+px)
	
End Function

Function UpdateTileMap(mesh%,sx%=0,sy%=0)
	
	Local x%,y%,u%,v%
	Local gfx%,gfxx%,gfxy%
	Local r%,g%,b%,a#
	Local surf%=GetSurface(mesh,1)
	
	For y%=0 To TilesY-1
		
		For x%=0 To TilesX-1
			
			gfxx=(sx+x) Mod MapSizeX
			gfxy=(sy+y)
			
			gfx=Normalize(Map(gfxx,gfxy),min,max,0,Terrains)
			
			u=(gfx Mod 4)*TileSize
			v=(gfx/4)*TileSize
			
			VertexTexCoords surf,V0(x,y),u,v
			VertexTexCoords surf,V1(x,y),u+TileSize,v
			VertexTexCoords surf,V2(x,y),u+TileSize,v+TileSize
			VertexTexCoords surf,V3(x,y),u,v+TileSize
			
		Next
		
	Next
	
End Function

Function CreateTileMap(sx%=0,sy%=0)
	
	Local Mesh%=CreateMesh()
	Local Surf%=CreateSurface(Mesh)
	
	Local x%,y%,u%,v%
	Local r%,g%,b%,a#
	Local gfx%,gfxx%,gfxy%
	
	For y%=0 To TilesY-1
		
		For x%=0 To TilesX-1
			
			V0(x,y)=AddVertex(Surf,x-0.5,y-0.5,0)
			V1(x,y)=AddVertex(Surf,x+0.5,y-0.5,0)
			V2(x,y)=AddVertex(Surf,x+0.5,y+0.5,0)
			V3(x,y)=AddVertex(Surf,x-0.5,y+0.5,0)
			
			AddTriangle(Surf,V0(x,y),V1(x,y),V2(x,y))
			AddTriangle(Surf,V2(x,y),V3(x,y),V0(x,y))
			
		Next
		
	Next
	
	ScaleMesh Mesh,TileSize,-TileSize,1
	PositionEntity Mesh,-(ScreenX/2.0)+(TileSize/2.0)-0.5,(Screeny/2.0)-(TileSize/2.0)+0.5,0
	
	Return Mesh
	
End Function
;~IDEal Editor Parameters:
;~C#Blitz3D