Import "minib3d.bmx"

SuperStrict

Incbin "planet.raw"
Incbin "tiles.png"

AppTitle="MiniB3D Tilemap"

Const ScreenX:Int		=	800
Const ScreenY:Int		=	608
Const TileSize:Int	=	32
Const TileColumns:Int	=	4
Const TileRows:Int	=	4
Const TilesX:Int		=	ScreenX/TileSize
Const TilesY:Int		=	ScreenY/TileSize
Const MapSizeX:Int	=	1024
Const MapSizeY:Int	=	512

Global Terrains:Int	=	(TileColumns*TileRows)

Global TileX:Int		=	256					' start position X
Global TileY:Int		=	256					' start position Y
Global Minimum:Int	=	255
Global Maximum:Int	=	0

Local sw:Int
Local old_ms:Int=MilliSecs()
Local renders:Int
Local fps:Int
Local mode:String="3D"

Global V0:Int[TilesX,TilesY],V1:Int[TilesX,TilesY],V2:Int[TilesX,TilesY],V3:Int[TilesX,TilesY]

Graphics3D ScreenX,ScreenY,32,2

' Load raw Heightmap to bank
Global Map:Int[MapsizeX,MapsizeY]
Local bank:TBank=LoadBank("incbin::planet.raw")
For Local x:Int=0 To MapSizeX-1
	For Local y:Int=0 To MapSizeY-1
		Map[x,y]=GetBankPixel(bank,x,y,MapSizeX)
		If Map[x,y]>Maximum Then Maximum=Map[x,y]
		If Map[x,y]<Minimum Then Minimum=Map[x,y]
	Next
Next

' Load Tile Image/Texture
Local tileimg:TImage=LoadImage("incbin::tiles.png",0)
Local tiletex:TTexture=LoadTexture("incbin::tiles.png",1+8)
Local TileTexWidth:Int=TextureWidth(tiletex)
Local TileTexHeight:Int=TextureHeight(tiletex)
ScaleTexture tiletex,TileTexWidth,TileTexHeight

' Create Tilemap
Local tilemap:TMesh=CreateTileMap()
UpdateTileMap(tilemap,TileX,TileY)
EntityTexture tilemap,tiletex,0,0
EntityFX tilemap,1+2

' Create Camera
Global Cam:TCamera=CreateCamera()
CameraRange Cam,1,ScreenX/2.0
PositionEntity Cam,0,0,-ScreenX/2.0

' Create Minimap
BeginMax2D()
Local minimap:TImage=CreateMinimap(bank,MapSizeX,MapSizeY)
EndMax2D()

While Not KeyDown(KEY_ESCAPE)

	Local cx:Int=KeyDown(KEY_RIGHT)-KeyDown(KEY_LEFT)
	Local cy:Int=KeyDown(KEY_DOWN)-KeyDown(KEY_UP)
	
	TileX=(TileX+cx) Mod MapSizeX
	If TileX<0 Then TileX=MapSizeX+TileX
	
	TileY=(TileY+cy)
	If TileY<0 Then TileY=0
	If TileY>MapSizeY-TilesY Then TileY=MapSizeY-TilesY

	If cx<>0 Or cy<>0 Then UpdateTileMap(tilemap,TileX,TileY)

	If KeyHit(KEY_SPACE) Then
		sw=1-sw
		If sw=1 Then mode="2D" Else mode="3D"
	EndIf
	
	RenderWorld

	BeginMax2D()

		If sw=1 Then

			SetColor 255,255,255
		
			For Local y:Int=0 To TilesY-1
		
				For Local x:Int=0 To TilesX-1
			
					Local mx:Int=(TileX+x) Mod MapSizeX
					Local my:Int=(TileY+y)
			
         				Local h:Int=Normalize(Map[mx,my],Minimum,Maximum,0,Terrains)
			
					Local u:Int=(h Mod TileColumns)
					Local v:Int=(h/TileRows)
								
					DrawSubImageRect tileimg,x*32,y*32,32,32,u*32,v*32,32,32
		
				Next
			
			Next
		
		EndIf

		Local stepx:Int=(MapSizeX/256)
		Local stepy:Int=(MapSizeY/128)
	
		SetColor 255,255,255
		DrawImage minimap,0,0
	
		SetColor 255,0,0
		SetBlend SOLIDBLEND
		DrawRect TileX/stepx,TileY/stepy,TilesX/Stepx,TilesY/stepy
			
	EndMax2D()
	
	renders=renders+1
	
	' calculate fps
	If MilliSecs()-old_ms>=1000
		old_ms=MilliSecs()
		fps=renders
		renders=0
	EndIf
	
	Text 0,128,mode+": "+fps+" FPS"
	
	Flip 0

Wend

End

Function CreateMinimap:TImage(bank:TBank,width:Int,height:Int)
	
	Local image:TImage=CreateImage(256,128)
	Local stepx:Int=width/256
	Local stepy:Int=height/128
	Local x:Int,y:Int,h:Int,rgb:Int
	
	Local pix:TPixmap
	pix=LockImage(image)
	
	For x=0 To 255
		
		For y=0 To 127
			
			h=Map[x*stepx,y*stepy]
			rgb=h*$1000000+h*$10000+h*$100+h
			WritePixel pix,x,y,rgb
			
		Next
		
	Next
	
	UnlockImage(image)
	
	Return image
	
End Function

Function CreateTileMap:TMesh(sx:Int=0,sy:Int=0)
	
	Local Mesh:Tmesh=CreateMesh()
 	Local Surf:TSurface=CreateSurface(Mesh)
	Local x:Int,y:Int

	For y=0 To TilesY-1
		
		For x=0 To TilesX-1
			
			V0[x,y]=AddVertex(Surf,x-0.5,y-0.5,0,0)
			V1[x,y]=AddVertex(Surf,x+0.5,y-0.5,0,0)
			V2[x,y]=AddVertex(Surf,x+0.5,y+0.5,0,0)
			V3[x,y]=AddVertex(Surf,x-0.5,y+0.5,0,0)
		
			AddTriangle(Surf,V0[x,y],V1[x,y],V2[x,y])
			AddTriangle(Surf,V2[x,y],V3[x,y],V0[x,y])
			
		Next
			
	Next
	
	ScaleMesh Mesh,TileSize,-TileSize,1
	PositionEntity Mesh,-(ScreenX/2.0)+(TileSize/2.0)-0.5,(Screeny/2.0)-(TileSize/2.0)+0.5,0
	
	Return Mesh
	
End Function

Function UpdateTileMap(mesh:Tmesh,sx:Int=0,sy:Int=0)
	
	Local x:Int,y:Int,u:Int,v:Int
	Local h:Int,mx:Int,my:Int
	Local r:Int,g:Int,b:Int,a:Float
	
	Local surf:TSurface=GetSurface(mesh,1)
	
	For y=0 To TilesY-1
		
		For x=0 To TilesX-1
			
         		mx=(sx+x) Mod MapSizeX
			my=(sy+y)
			
         		h=Normalize(Map[mx,my],Minimum,Maximum,0,Terrains)
			
			u=(h Mod TileColumns)*TileSize
			v=(h/TileRows)*TileSize
		
         		VertexTexCoords surf,V0[x,y],u,v
			VertexTexCoords surf,V1[x,y],u+TileSize,v
			VertexTexCoords surf,V2[x,y],u+TileSize,v+TileSize
			VertexTexCoords surf,V3[x,y],u,v+TileSize
					
		Next
		
	Next
	
End Function

Function Normalize:Float(value:Float=128.0,value_min:Float=0.0,value_max:Float=255.0,norm_min:Float=0.0,norm_max:Float=1.0)
	
	Return ((value-value_min)/(value_max-value_min))*(norm_max-norm_min)+norm_min
	
End Function

Function GetBankPixel:Int(bank:TBank,px:Int,py:Int,width:Int)
	
	Return PeekByte(bank,(py*width)+px)
	
End Function
